// Generated by CoffeeScript 1.10.0
(function() {
  var Promise, _deployAction, _forwardAction, _historyAction, _rollbackAction, chalk, cli, commander, fs, logger, path, pkg, sneaky;

  require('coffee-script/register');

  path = require('path');

  commander = require('commander');

  Promise = require('bluebird');

  fs = require('fs');

  logger = require('graceful-logger');

  chalk = require('chalk');

  pkg = require('../package');

  sneaky = require('./sneaky');

  ['Skyfile', 'skyfile', 'skyfile.js', 'Skyfile.js', 'skyfile.coffee', 'Skyfile.coffee'].some(function(fileName) {
    var err, error, filePath;
    filePath = path.join(process.cwd(), fileName);
    try {
      if (fs.existsSync(filePath)) {
        return require(filePath);
      }
    } catch (error) {
      err = error;
      logger.warn(err.message);
      return process.exit(1);
    }
  });

  _deployAction = function(taskName) {
    return Promise.resolve().then(function() {
      if (toString.call(taskName) !== '[object String]') {
        throw new Error("Invalid task name");
      }
    }).then(function() {
      return sneaky.getTask(taskName);
    }).then(function(task) {
      task.stdout.pipe(process.stdout);
      task.stderr.pipe(process.stderr);
      return task.deploy();
    })["catch"](function(err) {
      logger.warn(err.message);
      return process.exit(1);
    });
  };

  _historyAction = function(taskName) {
    return Promise.resolve().then(function() {
      if (toString.call(taskName) !== '[object String]') {
        throw new Error("Invalid task name");
      }
    }).then(function() {
      return sneaky.getTask(taskName);
    }).then(function(task) {
      task.stdout.pipe(process.stdout);
      task.stderr.pipe(process.stderr);
      return task.history();
    }).then(function(histories) {
      return histories.forEach(function(history) {
        if (history.current) {
          return console.log(chalk.green("* " + history.commit + "\t" + history.date));
        } else {
          return console.log("  " + history.commit + "\t" + history.date);
        }
      });
    })["catch"](function(err) {
      logger.warn(err.message);
      return process.exit(1);
    });
  };

  _rollbackAction = function(taskName, version) {
    if (arguments.length === 2) {
      version = 1;
    }
    return Promise.resolve().then(function() {
      if (toString.call(taskName) !== '[object String]') {
        throw new Error("Invalid task name");
      }
    }).then(function() {
      return sneaky.getTask(taskName);
    }).then(function(task) {
      task.stdout.pipe(process.stdout);
      task.stderr.pipe(process.stderr);
      return task.rollback(version);
    })["catch"](function(err) {
      logger.warn(err.message);
      return process.exit(1);
    });
  };

  _forwardAction = function(taskName, version) {
    if (arguments.length === 2) {
      version = 1;
    }
    return Promise.resolve().then(function() {
      if (toString.call(taskName) !== '[object String]') {
        throw new Error("Invalid task name");
      }
    }).then(function() {
      return sneaky.getTask(taskName);
    }).then(function(task) {
      task.stdout.pipe(process.stdout);
      task.stderr.pipe(process.stderr);
      return task.forward(version);
    })["catch"](function(err) {
      logger.warn(err.message);
      return process.exit(1);
    });
  };

  module.exports = cli = function() {
    commander.version(pkg.version, '-v, --version').usage('<command> taskName');
    commander.option('-T, --tasks', 'display the tasks').on('tasks', function() {
      var tasks;
      tasks = sneaky.getTasks();
      return Object.keys(tasks).forEach(function(key) {
        var task;
        task = tasks[key];
        return console.log(task.taskName + "\t\t" + (task.description || ''));
      });
    });
    commander.command('deploy').usage('taskName [options]').description('deploy application to server').action(_deployAction);
    commander.command('history').usage('taskName [options]').description('display previous deploy histories').action(_historyAction);
    commander.command('rollback').usage('taskName [version]').description('rollback to the previous version').action(_rollbackAction);
    commander.command('forward').usage('taskName [version]').description('roll forward to the later version, opposite of rollback').action(_forwardAction);
    commander.command('d').usage('taskName [options]').description('alias of deploy').action(_deployAction);
    commander.command('h').usage('taskName [options]').description('alias of history').action(_historyAction);
    commander.command('r').usage('taskName [version]').description('alias of rollback').action(_rollbackAction);
    commander.command('f').usage('taskName [version]').description('alias of forward').action(_forwardAction);
    commander.parse(process.argv);
    if (process.argv.length < 3) {
      return commander.help();
    }
  };

}).call(this);
