// Generated by CoffeeScript 1.10.0
(function() {
  var Promise, mkdirp, mkdirpAsync, moment, path, rimraf, rimrafAsync;

  path = require('path');

  Promise = require('bluebird');

  mkdirp = require('mkdirp');

  rimraf = require('rimraf');

  mkdirpAsync = Promise.promisify(mkdirp);

  rimrafAsync = Promise.promisify(rimraf);

  moment = require('moment');


  /**
   * Prepare for the working directory of deployment
   * @param  {Object} task - Task instance
   * @return {Promise}
   */

  module.exports = function(task) {
    var $chdir, $mkTmpDir, $sourceRepos, $srcPath, $targetPath, sourceRepos, srcPath, tmpdir;
    task.source || (task.source = process.cwd());
    task.version || (task.version = 'HEAD');
    tmpdir = path.join(process.env.HOME, '.sneaky', "" + (Date.now()));
    process.chdir(task.source);
    if (task.nochdir) {
      $srcPath = Promise.resolve(task.source);
    } else {
      srcPath = path.join(tmpdir, 'deploy');
      $mkTmpDir = rimrafAsync(tmpdir).then(function() {
        return mkdirpAsync(tmpdir);
      }).then(function() {
        return mkdirpAsync(srcPath);
      });
      if (/^(http|git|ssh)/.test(task.source)) {
        sourceRepos = path.join(tmpdir, 'git-cache');
        $sourceRepos = $mkTmpDir.then(function() {
          return task.execCmd("git clone " + task.source + " " + sourceRepos);
        }).then(function() {
          return process.chdir(sourceRepos);
        });
      } else {
        $sourceRepos = $mkTmpDir.then(function() {
          return task.source;
        });
      }
      $srcPath = $sourceRepos.then(function(sourceRepos) {
        return task.execCmd("git archive " + task.version + " --remote=" + sourceRepos + " --format=tar.gz | tar -xzf - -C " + srcPath).then(function() {
          return srcPath;
        });
      });
    }
    $srcPath = $srcPath.then(function(srcPath) {
      return task.srcPath = srcPath;
    });
    if (task.overwrite) {
      $targetPath = Promise.resolve().then(function() {
        var targetPath;
        return task.targetPath = targetPath = path.join(task.path, 'source');
      });
    } else {
      $targetPath = $srcPath.then(function() {
        return task.execCmd("git log " + task.version + " --decorate --oneline | head -n 1");
      }).then(function(log) {
        var commit, decorate, matches, targetPath, version;
        matches = log.match(/([0-9a-z]{7})\ (\((.*)\) )?/i);
        commit = matches[1];
        decorate = matches[3];
        version = false;
        if (decorate != null) {
          decorate.split(',').some(function(dec) {
            if (dec.trim().indexOf('tag') === 0) {
              return version = dec.split(':')[1].trim();
            }
          });
        }
        version || (version = commit);
        return task.targetPath = targetPath = path.join(task.path, moment().format('YYYYMMDDHHmmss') + '-' + version);
      });
    }
    return $chdir = Promise.all([$targetPath, $srcPath]).then(function(arg) {
      var srcPath, targetPath;
      targetPath = arg[0], srcPath = arg[1];
      return process.chdir(srcPath);
    });
  };

}).call(this);
