// Generated by CoffeeScript 1.10.0
(function() {
  var EventEmitter, Promise, Task, exec, fs, logger, moment, path, ref, spawn, stream,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty,
    slice = [].slice;

  path = require('path');

  stream = require('stream');

  ref = require('child_process'), exec = ref.exec, spawn = ref.spawn;

  EventEmitter = require('events').EventEmitter;

  fs = require('fs');

  logger = require('graceful-logger');

  moment = require('moment');

  Promise = require('bluebird');

  Promise.promisifyAll(fs);

  Task = (function(superClass) {
    extend(Task, superClass);

    function Task() {
      this._preHooks = {};
      this._postHooks = {};
      this._steps = [
        {
          stepName: 'prepare',
          executor: require('./steps/prepare')
        }, {
          stepName: 'transport',
          executor: require('./steps/transport')
        }
      ];
      this.stdout = new stream.Transform;
      this.stdout._transform = function(chunk, encoding, callback) {
        return callback(null, chunk);
      };
      this.stderr = new stream.Transform;
      this.stderr._transform = function(chunk, encoding, callback) {
        return callback(null, chunk);
      };
    }

    Task.prototype.initialize = function() {
      var task;
      task = this;
      task.appName || (task.appName = task.taskName.split(':')[0]);
      task.taskName || (task.taskName = path.basename(process.cwd()));
      task.source || (task.source = process.cwd());
      task.port || (task.port = 22);
      if (!task.path) {
        throw new Error("Task " + task.taskName + "'s path is undefined");
      }
    };


    /**
     * Execute this deploy process
     * @return {Promise} The task with the finishd state
     */

    Task.prototype.deploy = function() {
      var task;
      task = this;
      logger.info("start deploy " + task.taskName);
      return Promise.resolve().then(function() {
        return task._steps;
      }).each(function(step) {
        var $executor, _postHooks, _preHooks, executor, stepName;
        stepName = step.stepName, executor = step.executor;
        $executor = Promise.resolve();
        _preHooks = task._preHooks, _postHooks = task._postHooks;
        if (_preHooks[stepName]) {
          $executor = $executor.then(function() {
            return _preHooks[stepName];
          }).each(function(fn) {
            return fn.call(task, task);
          });
        }
        $executor = $executor.then(function() {
          return executor.call(task, task);
        });
        if (_postHooks[stepName]) {
          $executor = $executor.then(function() {
            return _postHooks[stepName];
          }).each(function(fn) {
            return fn.call(task, task);
          });
        }
        return $executor;
      }).then(function() {
        process.chdir(task.source);
        return logger.info("finish deploy " + task.taskName);
      });
    };

    Task.prototype.history = function() {
      var task;
      task = this;
      return Promise.resolve().then(function() {
        var sshCmd;
        sshCmd = task._wrapRemoteCmd("cd " + task.path + "; ls -l | tail -n 30");
        return new Promise(function(resolve, reject) {
          var child;
          return child = exec(sshCmd, function(err, out) {
            if (err) {
              return reject(err);
            }
            return resolve(out);
          });
        });
      }).then(function(out) {
        var current, histories;
        current = '';
        return histories = out.split('\n').map(function(line) {
          var matches;
          if (/current/.test(line)) {
            current = path.basename(line).trim();
          } else if (matches = line.match(/\d{14}\-.*/i)) {
            return matches[0].trim();
          }
          return false;
        }).filter(function(history) {
          return history;
        }).map(function(history) {
          var commit, date, m, ref1;
          ref1 = history.split('-'), date = ref1[0], commit = 2 <= ref1.length ? slice.call(ref1, 1) : [];
          m = moment(date, 'YYYYMMDDHHmmss');
          return {
            date: m.format('YYYY-MM-DD HH:mm:ss'),
            commit: commit.join('-'),
            current: current === history ? true : false,
            dirname: history,
            path: path.join(task.path, history)
          };
        });
      });
    };

    Task.prototype._rollto = function(version, direction) {
      var task;
      if (version == null) {
        version = 1;
      }
      if (direction == null) {
        direction = 'down';
      }
      task = this;
      if (!isNaN(Number(version))) {
        version = Number(version);
      }
      return task.history().then(function(histories) {
        var chosenHistory, cmd, diffNum, meetCurrent, totalCount;
        meetCurrent = false;
        chosenHistory = false;
        diffNum = 0;
        totalCount = histories.length;
        if (direction === 'down') {
          histories = histories.reverse();
        }
        histories.some(function(history, idx) {
          if (history.current) {
            meetCurrent = true;
          }
          if (!meetCurrent) {
            return;
          }
          if (version === 'edge') {
            if (idx === totalCount - 1) {
              chosenHistory = history;
              return true;
            }
          } else if (toString.call(version) === '[object String]') {
            if (history.commit.indexOf(version) > -1) {
              chosenHistory = history;
              return true;
            }
          } else if (toString.call(version) === '[object Number]') {
            if (version === 0) {
              chosenHistory = history;
              return true;
            }
            version -= 1;
          } else {
            throw new Error("Invalid version '" + version + "'");
          }
          return false;
        });
        if (!chosenHistory) {
          throw new Error("Can not find the rollback version '" + version + "'");
        }
        cmd = "cd " + task.path + "; ln -sfn " + chosenHistory.path + " " + (path.join(task.path, 'current'));
        return task.execRemoteCmd(cmd).then(function() {
          return task.targetPath = chosenHistory.path;
        });
      }).then(function() {
        var ref1, ref2;
        if ((ref1 = task._postHooks) != null ? (ref2 = ref1.transport) != null ? ref2.length : void 0 : void 0) {
          return Promise.resolve(task._postHooks.transport).each(function(fn) {
            return fn.call(task, task);
          });
        }
      });
    };

    Task.prototype.rollback = function(version) {
      if (version == null) {
        version = 1;
      }
      logger.info("start rollback " + this.taskName + " to version " + version);
      if (version === 'first') {
        version = 'edge';
      }
      return this._rollto(version, 'down');
    };

    Task.prototype.forward = function(version) {
      if (version == null) {
        version = 1;
      }
      logger.info("start forward " + this.taskName + " to version " + version);
      if (version === 'last') {
        version = 'edge';
      }
      return this._rollto(version, 'up');
    };


    /**
     * Register pre hooks on step
     * @param  {String}   stepName - Name of step
     * @param  {Function} fn - Hook function
     * @return {Promise}
     */

    Task.prototype.pre = function(stepName, fn) {
      var base;
      (base = this._preHooks)[stepName] || (base[stepName] = []);
      return this._preHooks[stepName].push(fn);
    };


    /**
     * Register post hooks on step
     * @param  {String}   stepName - Name of step
     * @param  {Function} fn - Hook function
     * @return {Function}
     */

    Task.prototype.post = function(stepName, fn) {
      var base;
      (base = this._postHooks)[stepName] || (base[stepName] = []);
      return this._postHooks[stepName].push(fn);
    };


    /**
     * Execute script before transport
     * It is an alias function of `execCmd` before `transport` step
     * @param  {String} stepName - Name of step
     * @param  {String|Function} script - Script string or function
     * @return {Function}
     */

    Task.prototype.before = function(stepName, script) {
      var fn;
      if (arguments.length === 1) {
        script = stepName;
        stepName = 'transport';
      }
      if (toString.call(script) === '[object Function]') {
        fn = script;
      } else {
        fn = (function(_this) {
          return function() {
            return _this.execCmd(script);
          };
        })(this);
      }
      return this.pre(stepName, fn);
    };


    /**
     * Execute remote script after transport and linking to the new directory
     * It is an alias function of `execRemoteCmd` after `transport` step
     * @param  {String} stepName - Name of step
     * @param  {String|Function} script - Script string or function
     * @return {Function}
     */

    Task.prototype.after = function(stepName, script) {
      var fn;
      if (arguments.length === 1) {
        script = stepName;
        stepName = 'transport';
      }
      if (toString.call(script) === '[object Function]') {
        fn = script;
      } else {
        fn = (function(_this) {
          return function() {
            return _this.execRemoteCmd(script);
          };
        })(this);
      }
      return this.post(stepName, fn);
    };

    Task.prototype._execCmd = function(cmd) {
      var task;
      task = this;
      return new Promise(function(resolve, reject) {
        var child;
        child = exec(cmd, function(err, out) {
          if (err) {
            return reject(err);
          }
          return resolve(out);
        });
        child.stdout.on('data', function(data) {
          return task.stdout.write(data);
        });
        return child.stderr.on('data', function(data) {
          return task.stderr.write(data);
        });
      });
    };

    Task.prototype._wrapRemoteCmd = function(cmd) {
      var sshCmd, task;
      task = this;
      if (cmd.indexOf('cd ') !== 0) {
        cmd = "cd " + task.targetPath + " && " + cmd;
      }
      sshCmd = "ssh";
      if (task.port) {
        sshCmd += " -p " + task.port;
      }
      if (task.key) {
        sshCmd += " -i " + task.key;
      }
      sshCmd += " " + task.user + "@" + task.host;
      sshCmd += " \"" + cmd + "\"";
      return sshCmd;
    };


    /**
     * Execute shell on localhost
     * @param  {String} cmd - Commmand string
     * @return {Promise}
     */

    Task.prototype.execCmd = function(cmd) {
      logger.info(cmd);
      return this._execCmd(cmd);
    };


    /**
     * Execute shell on remote servers
     * @param  {String} cmd - Commmand string
     * @return {Promise}
     */

    Task.prototype.execRemoteCmd = function(cmd) {
      var sshCmd;
      sshCmd = this._wrapRemoteCmd(cmd);
      logger.info(sshCmd);
      return this._execCmd(sshCmd);
    };

    return Task;

  })(EventEmitter);

  module.exports = Task;

}).call(this);
